'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (_ref) {
  var t = _ref.types;
  var template = _ref.template;

  function shouldRegisterBinding(binding) {
    var _binding$path = binding.path;
    var type = _binding$path.type;
    var node = _binding$path.node;

    switch (type) {
      case 'FunctionDeclaration':
      case 'ClassDeclaration':
      case 'VariableDeclaration':
        return true;
      case 'VariableDeclarator':
        var init = node.init;

        if (t.isCallExpression(init) && init.callee.name === 'require') {
          return false;
        }
        return true;
      default:
        return false;
    }
  }

  var DEFAULT_EXPORT = Symbol();
  return {
    visitor: {
      ExportDefaultDeclaration: function ExportDefaultDeclaration(path) {
        if (path.node.declaration.id) {
          return;
        }

        var id = path.scope.generateUidIdentifier('default');
        var expression = t.isExpression(path.node.declaration) ? path.node.declaration : t.toExpression(path.node.declaration);

        path.insertBefore(t.variableDeclaration('const', [t.variableDeclarator(id, expression)]));
        path.node.declaration = id;
        path.parent[DEFAULT_EXPORT] = id;
      },


      Program: {
        enter: function enter(_ref2) {
          var node = _ref2.node;

          node[DEFAULT_EXPORT] = null;
        },
        exit: function exit(_ref3, _ref4) {
          var node = _ref3.node;
          var scope = _ref3.scope;
          var file = _ref4.file;

          var registrations = [];

          for (var id in scope.bindings) {
            var binding = scope.bindings[id];
            if (shouldRegisterBinding(binding)) {
              registrations.push(buildRegistration({
                ID: binding.identifier,
                NAME: t.stringLiteral(id)
              }));
            }
          }

          if (node[DEFAULT_EXPORT]) {
            registrations.push(buildRegistration({
              ID: node[DEFAULT_EXPORT],
              NAME: t.stringLiteral('default')
            }));
          }

          node.body.push(buildTagger({
            FILENAME: t.stringLiteral(file.opts.filename),
            REGISTRATIONS: registrations
          }));
        }
      }
    }
  };
};

var _babelTemplate = require('babel-template');

var _babelTemplate2 = _interopRequireDefault(_babelTemplate);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var buildRegistration = (0, _babelTemplate2.default)('\n  tagSource(ID, NAME);\n');
var buildTagger = (0, _babelTemplate2.default)('\n(function () {\n  function tagSource(fn, localName) {\n    if (typeof fn !== "function") {\n      return;\n    }\n\n    if (fn.hasOwnProperty("__source")) {\n      return;\n    }\n\n    try {\n      Object.defineProperty(fn, "__source", {\n        enumerable: false,\n        configurable: true,\n        value: {\n          fileName: FILENAME,\n          localName: localName\n        }\n      });\n    } catch (err) {}\n  }\n  REGISTRATIONS\n})();\n');